;**********************************************************************

;       Titolo  : COPCOP3.asm
;       Utilizzo: Verificatore impulsi
;       Progetto: COPTRON
;       Software: Ing.Pompetti
;       Versione: 1.3
;         controllo micro C in caso di guasto
;         reset contatori in caso di chiusura micro B
;         reset errore al posizionamento della pistola, per tutti gli
;            errori
;         ritardo tra un impulso in uscita ed il successivo di circa 1mS
;       Data:     7-11-97
;       Clock:    8 MHz
;       Checksum:
;**********************************************************************
.LIST   1
.PL     66
.LINESIZE       131
.TITLE          "Copcop3"
.W_ON

npulseA .equ    100     ; numero impulsi per fare lampeggiare il led A
npulseB .equ    100     ; numero impulsi per fare lampeggiare il led B
maxdiff .equ    3       ; massima differenza tra il numero di impulsi del
                        ; canale 1 e 2 dello stesso encoder
                        ; NEL PROGRAMMA  FISSATO IL VALORE 3

Lpulse  .equ    4       ; lunghezza impulsi di uscita n. interrupt 264uS cad


; *********************************************************************
; EQUATES AND DATA SPACE DEFINITIONS
; *********************************************************************

; **************************
; PROCESSOR REGISTERS
; **************************
X       .DEF 080H,0FFH,0FFH     ; index registers
Y       .DEF 081H,0FFH,0FFH
V       .DEF 082H,0FFH,0FFH
W       .DEF 083H,0FFH,0FFH
A       .DEF 0FFH,0FFH,0FFH
DDRPA   .DEF 0C4H,0FFH,0FFH
DDRPB   .DEF 0C5H,0FFH,0FFH
ORPA    .DEF 0CCH,0FFH,0FFH
ORPB    .DEF 0CDH,0FFH,0FFH
DRPA    .DEF 0C0H,0FFH,0FFH
DRPB    .DEF 0C1H,0FFH,0FFH
IOR     .DEF 0C8H,0FFH,0FFH     ; interrupt option register
RDW     .DEF 0C9H,0FFH,0FFH     ; data window register (only read)
ADR     .DEF 0D0H,0FFH,0FFH
ADCR    .DEF 0D1H,0FFH,0FFH     ; A/D Interrupt control register
PSC     .DEF 0D2H,0FFH,0FFH
TCR     .DEF 0D3H,0FFH,0FFH
TSCR    .DEF 0D4H,0FFH,0FFH     ; timer status control register
HWDR    .DEF 0D8H,0FFH,0FFH     ; Digital watch dog register
DRAMBK  .DEF 0E8H,0FFH,0FFH

; IOR bit equates
IOR4    .equ    4       ; 1=Interrupt enabled - 0=Interrupt disabled
IOR5    .equ    5       ; 1=IRQ2 Rising Edge  - 0=IRQ2 Falling Edge
IOR6    .equ    6       ; 1=IRQ1 Low Level    - 0=IRQ1 Falling Edge

; TSCR bit equates
PSO     .equ    0       ; Prescaler Mux. Select
PS1     .equ    1       ; Prescaler Mux. Select
PS2     .equ    2       ; Prescaler Mux. Select
PSI     .equ    3       ; Prescaler Initialize Bit
DOUT    .equ    4       ; Data Output
TOUT    .equ    5       ; Timer Output Control
ETI     .equ    6       ; Enable Timer Interrupt
TMZ     .equ    7       ; Timer Zero Bit

; ADCR bit register
PDS     .equ    4       ; Power Down Selection
STA     .equ    5       ; Start of Conversion
EOC     .equ    6       ; End of Conversion


; **************************
; User RAM: 60 bytes
; Begin Address: 084h
; End Address:   0BFh
; **************************
status  .def    084h,0FFh,0FFh  ;
contA1  .def    085h,0FFh,0FFh  ; contatore per impulsi encoder A ch 1
contA2  .def    086h,0FFh,0FFh  ; contatore per impulsi encoder A ch 2
contB1  .def    087h,0FFh,0FFh  ; contatore per impulsi encoder B ch 1
contB2  .def    088h,0FFh,0FFh  ; contatore per impulsi encoder B ch 2
contOUT .def    089h,0FFh,0FFh  ; contatore somma per uscita
flag    .def    08Ah,0FFh,0FFh  ; contiene flag vari
cledA   .def    08Bh,0FFh,0FFh  ; contatore per accensione led A
cledB   .def    08Ch,0FFh,0FFh  ; contatore per accensione led B
ms      .def    08Dh,0FFh,0FFh  ;
fase    .def    08Eh,0FFh,0FFh  ; flag per costruzione impulso in uscita
contldA .def    08Fh,0FFh,0FFh  ; contatore per tempo accensione led A
contldB .def    090h,0FFh,0FFh  ; contatore per tempo accensione led B
Icount  .def    091h,0FFh,0FFh  ; divisore in TIMER per operaz. a tempi lunghi
OLDB    .def    092h,0FFh,0FFh  ; contiene vecchi valori della porta DRPB
TimRlK  .def    093h,0FFh,0FFh  ;
TimRlKh .def    094h,0FFh,0FFh  ;
TimOuC  .def    095h,0FFh,0FFh  ;
cOUTA   .def    096h,0FFh,0FFh  ;
cOUTB   .def    097h,0FFh,0FFh  ;
us      .def    098h,0FFh,0FFh  ;

; **************************
; User bit equates
; **************************

; PORT A bit equates
faultA  .equ    0       ; led errore canale A
faultB  .equ    1       ; led errore canale B
microA  .equ    2       ; interruttore A
microB  .equ    3       ; interruttore B

; PORT B bit equates
ch1A    .equ    0       ; encoder A canale 1
ch2A    .equ    1       ; encoder A canale 2
ch1B    .equ    2       ; encoder B canale 1
ch2B    .equ    3       ; encoder B canale 2

out1A   .equ    4       ; uscita A canale 1
out1B   .equ    5       ; uscita B canale 1

outMicC .equ    6       ; out micro C
releK1  .equ    7       ; rele K1

; flag bit equates
ableA1  .equ    0       ; encoder A canale 1
ableA2  .equ    1       ; encoder A canale 2
ableB1  .equ    2       ; encoder B canale 1
ableB2  .equ    3       ; encoder B canale 2
abledA  .equ    4       ; abilita accensione led A
abledB  .equ    5       ; abilita accensione led B
inner   .equ    6       ; abilita uscita impulso
first   .equ    7       ; prima volta micro B estratto




.ORG 880H
;*********************************************
;          POWER ON RESET
;*********************************************
;       HARDWARE INIT MODULE
;*********************************************
Reset
        ldi     HWDR,0FFh       ; init hardware watch dog

; Port A specification:

        ldi     ORPA,00000011b
        ldi     DRPA,00000011b
        ldi    DDRPA,00000011b

; Port B specifications:
        ldi     ORPB,11110000b
        ldi     DRPB,00000000b
        ldi    DDRPB,11110000b

; Select timer operating mode

        ldi     TSCR,01101100b  ; Prescaler / 16
;        ldi     TCR,21          ; Timer ogni 504 uS
        ldi     TCR,11          ; Timer ogni 264 uS
        reti

        call    CLR_RAM         ; clear RAM bank

        ldi     fase,00000001b
        ldi     cledA,npulseA
        ldi     cledB,npulseB
        set     ableA1,flag
        set     ableA2,flag
        set     ableB1,flag
        set     ableB2,flag
        ldi     Icount,4
        ldi     status,0
        ldi     IOR,50h

;...PROGRAMMA PRINCIPALE
loop
; verifica l'arrivo di un qualche impulso
       ldi      ms,1
       call     delay              ;[*]
       ldi      HWDR,0FFh           ; reload hardware watch dog
       ld       A,DRPB
       com      A
       andi     A,00001111b
       cp       A,OLDB
       jrz      main3
       call     readDRPB        ; impulso arrivato !
main3
;... verifica che la differenza tra gli impulsi dei due canali degli
;    encoder sia minore di maxdiff
       ld       A,contA1
       sub      A,contA2
       jrr      7,A,main4
       dec      A
       com      A
main4
       andi     A,11111100b
       jrz      main5
       res      faultA,DRPA
       call     guasto
main5
       ld       A,contB1
       sub      A,contB2
       jrr      7,A,main6
       dec      A
       com      A
main6
       andi     A,11111100b
       jrz      main7
       res      faultB,DRPA
       call     guasto                  ; <<< --- verificare
main7
       jrs      microB,DRPA,main700

       jrr      first,flag,ctrl0
       res      first,flag
       jp       ctrl5
ctrl0

; controllo che uno dei due encoder non abbia alberino rotto

       ld       A,contA1
       cpi      A,200
       jrz      ctrl1
       jp       ctrl3
ctrl1
       ld       A,contB1
       cpi      A,100
       jrnc     ctrl2
;... encoder B1 non gira abbastanza
       jp       guastoB

ctrl2
       jp       ctrl5           ; clear and exit

ctrl3
       ld       A,contB1
       cpi      A,200
       jrz      ctrl4
       jp       main700
ctrl4
       ld       A,contA1
       cpi      A,100
       jrnc     ctrl5
;... encoder A1 non gira abbastanza
       jp       guastoA
ctrl5
       ldi      contA1,0
       ldi      contB1,0
       ldi      contA2,0
       ldi      contB2,0
       jp       main700

main699
       set      first,flag



main700
       ld       A,status
       cpi      A,0
       jrz      main70
       jp       main8
main70
       jrs      microA,DRPA,main71
       jrs      microA,DRPA,main71
       jrs      microA,DRPA,main71
       ldi      status,1
       jp       main8
main71
       jrs      microB,DRPA,main72
       jrs      microB,DRPA,main72
       jrs      microB,DRPA,main72
       ldi      status,2
       jp       main9
main72
       jp       loop

main8
       ld       A,status
       cpi      A,1
       jrnz     main9
       set      outMicC,DRPB
       jrr      microA,DRPA,main81
       res      outMicC,DRPB
       ldi      status,8
main81
       jp       loop

main9
       ld       A,status
       cpi      A,2
       jrnz     mainA
       set      releK1,DRPB
       ldi      TimOuC,100
       ldi      status,3
       jp       loop
mainA
       ld       A,status
       cpi      A,3
       jrnz     mainB
       ld       A,TimOuC
       cpi      A,0
       jrnz     mainA1
       jp       loop
mainA1
       set      outMicC,DRPB
       ldi      status,4
       jp       loop
mainB
       ld       A,status
       cpi      A,4
       jrnz     mainC
       jrr      microB,DRPA,mainB1
       res      outMicC,DRPB
       ldi      status,5
       ldi      TimRlK,255
mainB1
       jp       loop
mainC
       ld       A,status
       cpi      A,5
       jrnz     mainD
       ld       A,TimRlK
       cpi      A,0
       jrnz     mainC1
       res      releK1,DRPB
       ldi      status,8
mainC1
       jp       loop


mainD
       ld       A,status
       cpi      A,8
       jrnz     mainE

       jrr      microB,DRPA,mainE
       jrr      microA,DRPA,mainE
       ldi      status,0

mainE
       jp       loop



;*****************************************************************************
;*       SUBROUTINES DI USO COMUNE                                           *
;*****************************************************************************

overflow
; avviene quando gli impulsi in ingresso sono troppo veloci
; per potere essere smaltiti in uscita

        res     faultA,DRPA     ; accende i due led di errore
        res     faultB,DRPA

guasto
        ldi     IOR,0
        ldi     TSCR,0
        set     out1A,DRPB
        set     out1B,DRPB
        ldi      HWDR,0FFh           ; reload hardware watch dog
        jrr     microA,DRPA,guasto      ;[*]
        jrr     microB,DRPA,guasto      ;[*]
rst                                     ; [*] reset
        jp      rst


guastoA
        set     out1A,DRPB
        ldi     IOR,0
        ldi     TSCR,0
guastoA1
        call    CheckB
        res     faultA,DRPA
        ldi     ms,240           ; reload hardware watch dog
        call    delay                   ;[*]
        call    CheckB
        ldi     ms,80            ; reload hardware watch dog
        call    delay
        call    CheckB
        set     faultA,DRPA
        ldi     ms,240           ; reload hardware watch dog
        call    delay
        call    CheckB
        ldi     ms,80            ; reload hardware watch dog
        call    delay

        jrr     microA,DRPA,guastoA1    ;[*]
        jrr     microB,DRPA,guastoA1    ;[*]
        jp      rst

guastoB
        set     out1B,DRPB
        ldi     IOR,0
        ldi     TSCR,0
guastoB1
        call    CheckB
        res     faultB,DRPA
        ldi     ms,80            ; reload hardware watch dog
        call    delay                   ;[*]
        call    CheckB
        ldi     ms,240           ; reload hardware watch dog
        call    delay                   ;[*]
        set     faultB,DRPA
        call    CheckB
        ldi     ms,80            ; reload hardware watch dog
        call    delay
        call    CheckB
        ldi     ms,240           ; reload hardware watch dog
        call    delay

        jrr     microA,DRPA,guastoB1    ;[*]
        jrr     microB,DRPA,guastoB1    ;[*]
        jp      rst

; ***********************************************
CheckB
       jrr      microB,DRPA,check1
       res      outMicC,DRPB

check1
        ret


readDRPB
;... elaborazione impulsi encoders

       ld      OLDB,A           ; salva nuovo valore porta B

;... verifica che l'encoder A non abbia entrambe le uscite basse
       ld       A,DRPB
;       com      A
       andi     A,00000011b
       jrnz     r00
       res      faultA,DRPA  ; errore... encoder A entrambe le uscite sono basse
       call     guasto
r00
       ld       A,DRPB
;       com      A
       andi     A,00001100b
       jrnz     r01
       res      faultB,DRPA  ; errore... encoder B entrambe le uscite sono basse
       call     guasto
r01

;... ripristina i canali che sono tornati alti

        jrs     ch1A,OLDB,r1
        set     ableA1,flag
r1
        jrs     ch2A,OLDB,r2
        set     ableA2,flag
r2
        jrs     ch1B,OLDB,r3
        set     ableB1,flag
r3
        jrs     ch2B,OLDB,r4
        set     ableB2,flag
r4

;... conta gli impulsi pervenuti ai canali abilitati

        jrr     ableA1,flag,ipb2
        jrr     ch1A,OLDB,ipb2
        res     ch2A,OLDB           ;[*]
        ldi     cOUTA,Lpulse
        set     out1A,DRPB
        inc     contA1
        dec     cledA
        jrnz    ipb11
        ldi     cledA,npulseA
        set     abledA,flag
ipb11
        inc     contOUT
        res     ableA1,flag
        jrnz    ipb2
        call    overflow
ipb2
        jrr     ableA2,flag,ipb3
        jrr     ch2A,OLDB,ipb3
        res     ableA1,flag
        ldi     cOUTB,Lpulse
        set     out1B,DRPB
        inc     contA2
        res     ableA2,flag
ipb3
        jrr     ableB1,flag,ipb4
        jrr     ch1B,OLDB,ipb4
        inc     contB1

        dec     cledB
        jrnz    ipb31
        ldi     cledB,npulseB
        set     abledB,flag
ipb31

        inc     contOUT
        res     ableB1,flag
        jrnz    ipb4
        call    overflow
ipb4
        jrr     ableB2,flag,ipb5
        jrr     ch2B,OLDB,ipb5
        inc     contB2
        res     ableB2,flag
ipb5

ipbend
        ret


; ***********************************************
; Name: Delay   - Ritardo di ms x 4143.75 microsecondi
; Entry: ms
; Exit: none
; Scratch: ms,us
; ***********************************************
delay
                ldi     us,64          ; [*]

loop100         ldi     HWDR,0FFh       ; reload hardware watch dog
                dec     us              ; 16.25 uS per ciclo
                jrnz    loop100

                dec     ms
                jrnz    delay
                ret





;****************************************************
;* Nome:  CLR_RAM
;* Utilizzo: Questa routine cancella tutte le ram dati ed i flag
;****************************************************

CLR_RAM
         ldi   X,0C0h                 ;
clr_ral  dec   X                      ;
         sub   A,A                    ;
         ld    (X),A                  ;
         ld    A,X                    ;
         jrnz  clr_ral                ;
         ret                          ;
;*****************************************************
;      ROUTINES DI SERVIZIO DEGLI INTERRUPTS                                  *
;*****************************************************



;**************************************************
; Name:  Timer  -  Timer Interrupt Service Routine
;**************************************************
;... timer generato ogni 504 uS

Timer
        ldi     TCR,11          ; Timer ogni 264 uS

;        ldi   TCR,21
        res   TMZ,TSCR          ; clear interrupt source
;... verifica se ci sono impulsi appesi da fare uscire
        inc     cOUTA
        dec     cOUTA
        jrnz    tim1
        res     out1A,DRPB
tim1
        dec     cOUTA

        inc     cOUTB
        dec     cOUTB
        jrnz    tim2
        res     out1B,DRPB
tim2
        dec     cOUTB


        dec     Icount
        jrz     tim3
        reti
tim3
;... routine eseguita ogni 2016 uS

        ldi     Icount,8

        inc     TimOuC
        dec     TimOuC
        jrnz    tim8
        jp      tim10
tim8
        dec     TimOuC
tim10
        dec     TimRlKh
        jrz     tim11
        reti
tim11
        ldi     TimRlKh,2

        inc     TimRlK
        dec     TimRlK
        jrz     tim12
        dec     TimRlK
tim12

        reti
;------------------------------------------------------------------------------



Ipa
Nmi
Ipb
Iad    reti

;**********************************************
;*     RESET   AND   INTERRUPT   VECTORS
;**********************************************

        .org    0FF0h
        jp      Iad            ; A/D interrupt vector
        jp      Timer           ; Timer interrupt vector
        jp      Ipb             ; Port B interrupt vector
        jp      Ipa             ; Port A interrupt vector
        .org    0FFCh
        jp      Nmi             ; NMI interrupt vector
        jp      Reset           ; Reset

